from typing import Any

version = ...  # type: Any

class ErrorHandler:
    def error(self, exception): ...
    def fatalError(self, exception): ...
    def warning(self, exception): ...

class ContentHandler:
    def __init__(self) -> None: ...
    def setDocumentLocator(self, locator): ...
    def startDocument(self): ...
    def endDocument(self): ...
    def startPrefixMapping(self, prefix, uri): ...
    def endPrefixMapping(self, prefix): ...
    def startElement(self, name, attrs): ...
    def endElement(self, name): ...
    def startElementNS(self, name, qname, attrs): ...
    def endElementNS(self, name, qname): ...
    def characters(self, content): ...
    def ignorableWhitespace(self, whitespace): ...
    def processingInstruction(self, target, data): ...
    def skippedEntity(self, name): ...

class DTDHandler:
    def notationDecl(self, name, publicId, systemId): ...
    def unparsedEntityDecl(self, name, publicId, systemId, ndata): ...

class EntityResolver:
    def resolveEntity(self, publicId, systemId): ...

feature_namespaces = ...  # type: Any
feature_namespace_prefixes = ...  # type: Any
feature_string_interning = ...  # type: Any
feature_validation = ...  # type: Any
feature_external_ges = ...  # type: Any
feature_external_pes = ...  # type: Any
all_features = ...  # type: Any
property_lexical_handler = ...  # type: Any
property_declaration_handler = ...  # type: Any
property_dom_node = ...  # type: Any
property_xml_string = ...  # type: Any
property_encoding = ...  # type: Any
property_interning_dict = ...  # type: Any
all_properties = ...  # type: Any
