from typing import Any

ssl = ...  # type: Any

class TCPServer:
    io_loop = ...  # type: Any
    ssl_options = ...  # type: Any
    max_buffer_size = ...  # type: Any
    read_chunk_size = ...  # type: Any
    def __init__(self, io_loop=..., ssl_options=..., max_buffer_size=..., read_chunk_size=...) -> None: ...
    def listen(self, port, address=...): ...
    def add_sockets(self, sockets): ...
    def add_socket(self, socket): ...
    def bind(self, port, address=..., family=..., backlog=...): ...
    def start(self, num_processes=...): ...
    def stop(self): ...
    def handle_stream(self, stream, address): ...
