from typing import Any

class UserAgentParser:
    platforms = ...  # type: Any
    browsers = ...  # type: Any
    def __init__(self): ...
    def __call__(self, user_agent): ...

class UserAgent:
    string = ...  # type: Any
    def __init__(self, environ_or_string): ...
    def to_header(self): ...
    def __nonzero__(self): ...
    __bool__ = ...  # type: Any
