from typing import Any

deque = ...  # type: Any
missing = ...  # type: Any
RegexType = ...  # type: Any
HELP_HTML = ...  # type: Any
OBJECT_DUMP_HTML = ...  # type: Any

def debug_repr(obj): ...
def dump(obj=...): ...

class _Helper:
    def __call__(self, topic=None): ...

helper = ...  # type: Any

class DebugReprGenerator:
    def __init__(self): ...
    list_repr = ...  # type: Any
    tuple_repr = ...  # type: Any
    set_repr = ...  # type: Any
    frozenset_repr = ...  # type: Any
    deque_repr = ...  # type: Any
    def regex_repr(self, obj): ...
    def string_repr(self, obj, limit=70): ...
    def dict_repr(self, d, recursive, limit=5): ...
    def object_repr(self, obj): ...
    def dispatch_repr(self, obj, recursive): ...
    def fallback_repr(self): ...
    def repr(self, obj): ...
    def dump_object(self, obj): ...
    def dump_locals(self, d): ...
    def render_object_dump(self, items, title, repr=None): ...
